@@ -1,5 +1,5 @@
 {- hpodder component
-Copyright (C) 2006-2007 John Goerzen <jgoerzen@complete.org>
+Copyright (C) 2006-2008 John Goerzen <jgoerzen@complete.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 {- |
    Module     : DB
-   Copyright  : Copyright (C) 2006-2007 John Goerzen
+   Copyright  : Copyright (C) 2006-2008 John Goerzen
    License    : GNU GPL, version 2 or above
 
    Maintainer : John Goerzen <jgoerzen@complete.org>
@@ -69,7 +69,38 @@ prepSchema dbh tables =
                commit dbh
                return 0
 
-upgradeSchema dbh 4 _ = return ()
+upgradeSchema dbh 5 _ = return ()
+
+upgradeSchema dbh 4 tables =
+    do dbdebug "Upgrading schema 4 -> 5"
+       dbdebug "Recreating episodes table to add epguid column and UNIQUE constaint"
+       -- Silly sqlite can't add a UNIQUE constaint to an existing table, so we
+       -- have to recreate it.
+       run dbh "CREATE TABLE episodes5 (\
+               \castid INTEGER NOT NULL,\
+               \episodeid INTEGER NOT NULL,\
+               \title TEXT NOT NULL,\
+               \epurl TEXT NOT NULL,\
+               \enctype TEXT NOT NULL,\
+               \status TEXT NOT NULL,\
+               \eplength INTEGER NOT NULL DEFAULT 0,\
+               \epfirstattempt INTEGER,\
+               \eplastattempt INTEGER,\
+               \epfailedattempts INTEGER NOT NULL DEFAULT 0,\
+               \epguid TEXT,\
+               \UNIQUE(castid, epurl),\
+               \UNIQUE(castid, episodeid),\
+               \UNIQUE(castid, epguid))" []
+       dbdebug "Copying data from old episodes table"
+       run dbh "INSERT INTO episodes5 SELECT *, NULL FROM episodes" []
+       dbdebug "Dropping old episodes table"
+       run dbh "DROP TABLE episodes" []
+       dbdebug "Renaming new episodes table"
+       run dbh "ALTER TABLE episodes5 RENAME TO episodes" []
+
+       setSchemaVer dbh 5
+       commit dbh
+       upgradeSchema dbh 5 tables
 
 upgradeSchema dbh 3 tables =
     do dbdebug "Upgrading schema 3 -> 4"
@@ -87,6 +118,7 @@ upgradeSchema dbh 3 tables =
 
        setSchemaVer dbh 4
        commit dbh
+       upgradeSchema dbh 4 tables
 
 upgradeSchema dbh 2 tables =
     do dbdebug "Upgrading schema 2 -> 3"
@@ -202,12 +234,12 @@ getEpisodes :: Connection -> Podcast -> IO [Episode]
 getEpisodes dbh pc =
     do r <- quickQuery dbh "SELECT episodeid, title, epurl, enctype,\
                             \status, eplength, epfirstattempt, eplastattempt, \
-                            \epfailedattempts FROM episodes \
+                            \epfailedattempts, epguid FROM episodes \
                             \WHERE castid = ? ORDER BY \
                             \episodeid" [toSql (castid pc)]
        return $ map toItem r
     where toItem [sepid, stitle, sepurl, senctype, sstatus, slength,
-                  slu, sla, sfa] =
+                  slu, sla, sfa, sepguid] =
               Episode {podcast = pc, epid = fromSql sepid,
                        eptitle = fromSql stitle,
                        epurl = fromSql sepurl, eptype = fromSql senctype,
@@ -215,7 +247,8 @@ getEpisodes dbh pc =
                        eplength = fromSql slength,
                        epfirstattempt = fromSql slu,
                        eplastattempt = fromSql sla,
-                       epfailedattempts = fromSql sfa}
+                       epfailedattempts = fromSql sfa,
+                       epguid = fromSql sepguid}
           toItem x = error $ "Unexpected result in getEpisodes: " ++ show x
 
 podcast_convrow [svid, svname, svurl, isenabled, lupdate, lattempt,
@@ -225,11 +258,28 @@ podcast_convrow [svid, svname, svurl, isenabled, lupdate, lattempt,
              lastupdate = fromSql lupdate, lastattempt = fromSql lattempt,
              failedattempts = fromSql fattempts}
 
-{- | Add a new episode.  If the episode already exists, ignore the add request
-and preserve the existing record. -}
+{- | Add a new episode. If the episode already exists, based solely on
+looking at the GUID (if present), update the URL and title fields while 
+preserving other fields as they are. Returns the number of inserted rows. -}
 addEpisode :: Connection -> Episode -> IO Integer
 addEpisode dbh ep = 
-    do nextepid <- getepid
+    do 
+       -- We have to be careful of cases where a feed may have two
+       -- different episodes with different GUIDs but identical URLs.
+       -- So if we have a GUID match here, we must have a conflict on URL,
+       -- so we ignore the request to change it.
+       when (epguid ep /= Nothing) $
+          do run dbh "UPDATE OR IGNORE episodes SET epurl = ?, epguid = ?, title = ? \
+                  \WHERE castid = ? AND epguid = ?"
+                 [toSql (epurl ep), toSql (epguid ep), toSql (eptitle ep),
+                  toSql (castid (podcast ep)), toSql (epguid ep)]
+             return ()
+       -- if the UPDATE was successful, that means that something with the same
+       -- URL or GUID already exists, so the INSERT below will be ignored.
+       dbdebug "update done"
+       nextepid <- getepid
+       dbdebug $ "addEpisode: epid: " ++ show nextepid
+       dbdebug "addEpisode: running insertEpisode"
        insertEpisode "INSERT OR IGNORE" dbh ep nextepid
     where getepid = 
               do r <- quickQuery dbh "SELECT MAX(episodeid) FROM episodes WHERE castid = ?" [toSql (castid (podcast ep))]
@@ -245,12 +295,13 @@ updateEpisode dbh ep = insertEpisode "INSERT OR REPLACE" dbh ep (epid ep)
 insertEpisode insertsql dbh episode newepid =
     run dbh (insertsql ++ " INTO episodes (castid, episodeid, title,\
            \epurl, enctype, status, eplength, epfirstattempt, eplastattempt,\
-           \epfailedattempts) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
+           \epfailedattempts, epguid) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
            [toSql (castid (podcast episode)), toSql newepid, 
             toSql (eptitle episode), toSql (epurl episode), 
             toSql (eptype episode), toSql (show (epstatus episode)),
             toSql (eplength episode), toSql (epfirstattempt episode),
-            toSql (eplastattempt episode), toSql (epfailedattempts episode)]
+            toSql (eplastattempt episode), toSql (epfailedattempts episode),
+            toSql (epguid episode)]
 
 getSelectedPodcasts dbh [] = getSelectedPodcasts dbh ["all"]
 getSelectedPodcasts dbh ["all"] = getPodcasts dbh
