@@ -1,5 +1,5 @@
 {- hpodder component
-Copyright (C) 2006 John Goerzen <jgoerzen@complete.org>
+Copyright (C) 2006-2007 John Goerzen <jgoerzen@complete.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 {- |
    Module     : Download
-   Copyright  : Copyright (C) 2006 John Goerzen
+   Copyright  : Copyright (C) 2006-2007 John Goerzen
    License    : GNU GPL, version 2 or above
 
    Maintainer : John Goerzen <jgoerzen@complete.org>
@@ -30,7 +30,7 @@ Written by John Goerzen, jgoerzen\@complete.org
 -}
 
 module Download(startGetURL, finishGetURL, checkDownloadSize, Result(..), 
-                DownloadTok(..)) where
+                DownloadTok(..), getdlfname) where
 import System.Cmd.Utils
 import System.Posix.Process
 import Config
@@ -45,7 +45,7 @@ import System.Posix.IO
 import Data.Hash.MD5
 import Control.Exception(evaluate)
 
-data Result = Success | TempFail | PermFail
+data Result = Success | Failure
             deriving (Eq, Show, Read)
 
 data DownloadTok = 
@@ -88,7 +88,7 @@ startGetURL url dirbase allowresume =
        havecurlrc <- doesFileExist curlrc
        let curlrcopts = (if havecurlrc then ["-K", curlrc] else [])
                         ++ (if allowresume then ["-C", "-"] else [])
-       let fp = dirbase ++ "/" ++ md5s (Str url)
+       let fp = dirbase ++ "/" ++ getdlfname url
        startsize <- getsize fp
        case startsize of 
          Just x -> d $ printf "Resuming download of %s at %s" fp (show x)
@@ -104,6 +104,7 @@ startGetURL url dirbase allowresume =
        closeFd msgfd2
        return $ DownloadTok pid url fp startsize
 
+getdlfname url = md5s (Str url)
 {- | Checks to see how much has been downloaded on the given file.  Also works
 after download is complete to get the final size.  Returns Nothing if the
 file doesn't exist. -}
@@ -115,24 +116,9 @@ finishGetURL dltok ec =
     do newsize <- getsize (tokpath dltok)
        let r = case ec of
                   Exited ExitSuccess -> Success
-                  Exited (ExitFailure i) ->
-                      case i of
-                        5 -> TempFail -- error resolving proxy
-                        6 -> TempFail -- error resolving host
-                        7 -> TempFail -- couldn't connect
-                        9 -> TempFail -- FTP access denied
-                        15 -> TempFail -- FTP host problem
-                        16 -> TempFail -- FTP connect problem
-                        18 -> TempFail -- Partial transfer
-                        26 -> TempFail -- Read error
-                        27 -> TempFail -- out of memory
-                        28 -> TempFail -- timeout
-                        35 -> TempFail -- SSL handshake fail
-                        37 -> TempFail -- permissions
-                        52 -> TempFail -- no reply
-                        _ -> PermFail
-                  Terminated _ -> TempFail
-                  Stopped _ -> TempFail
+                  Exited (ExitFailure i) -> Failure
+                  Terminated _ -> Failure
+                  Stopped _ -> Failure
        if r == Success
           then do d $ "curl returned successfully; new size is " ++
                         (show newsize)
@@ -142,11 +128,11 @@ finishGetURL dltok ec =
                      then do i $ "Attempt to resume download failed; will re-try download on next run"
                              removeFile (tokpath dltok)
                              --getURL url fp
-                             return TempFail
+                             return Failure
                      else if newsize == Nothing
                              -- Sometimes Curl returns success but doesn't 
                              -- actually download anything
-                             then return TempFail
+                             then return Failure
                              else return r
           else do d $ "curl returned error; new size is " ++ (show newsize)
                   return r
